@@ -64,13 +64,31 @@ namespace CtTextIterUtil {
 
 bool get_is_camel_case(Gtk::TextIter iter_start, int num_chars);
 
-bool get_first_chars_of_string_are(const Glib::ustring& text, const std::vector<Glib::ustring>& chars_list);
+inline bool startswith(Gtk::TextIter text_iter, const gchar* str)
+{
+    gunichar ch = g_utf8_get_char(str);
+    while (true)
+    {
+        if (text_iter.get_char() != ch)
+            return false;
+        str = g_utf8_next_char(str);
+        ch = g_utf8_get_char(str);
+        if (ch == 0)
+            return true;
+        if (!text_iter.forward_char())
+            return false;
+    }
+}
 
-bool get_next_chars_from_iter_are(Gtk::TextIter text_iter, const Glib::ustring& chars_list);
 
-bool get_next_chars_from_iter_are(Gtk::TextIter text_iter, const std::vector<Glib::ustring>& chars_list_vec);
-
-bool get_first_chars_of_string_at_offset_are(const Glib::ustring& in_string, int offset, const std::vector<Glib::ustring>& chars_list_vec);
+template<class container>
+bool startswith_any(Gtk::TextIter text_iter, const container& str_list)
+{
+    for (auto it = std::begin(str_list); it != std::end(str_list); ++it)
+        if (startswith(text_iter, *it))
+            return true;
+    return false;
+}
 
 void rich_text_attributes_update(const Gtk::TextIter& text_iter, std::map<const gchar*, std::string>& curr_attributes);
 
@@ -83,6 +101,8 @@ void generic_process_slot(int start_offset,
 
 const gchar* get_text_iter_alignment(const Gtk::TextIter& textIter, CtMainWin* pCtMainWin);
 
+int get_words_count(const Glib::RefPtr<Gtk::TextBuffer>& text_buffer);
+
 } // namespace CtTextIterUtil
 
 namespace CtStrUtil {
@@ -95,24 +115,38 @@ guint32 guint32_from_hex_chars(const char* hexChars, guint8 numChars);
 
 std::vector<gint64> gstring_split_to_int64(const gchar* inStr, const gchar* delimiter, gint max_tokens=-1);
 
-template<class IterableOfPgchar>
-bool is_pgchar_in_pgchar_iterable(const gchar* pGcharNeedle, const IterableOfPgchar& haystackOfPgchar)
+template<class container>
+bool contains(const container& array, const gchar* el)
 {
-    bool gotcha{false};
-    for (const gchar* pGcharHaystack : haystackOfPgchar)
-    {
-        if (0 == g_strcmp0(pGcharHaystack, pGcharNeedle))
-        {
-            gotcha = true;
-            break;
+    for (auto it = std::begin(array); it != std::end(array); ++it)
+        if (0 == g_strcmp0(*it, el))
+            return true;
+    return false;
+}
+
+template<class String>
+bool contains_words(const String& text, const std::vector<String>& words, bool require_all = true)
+{
+    for (auto& word: words) {
+        if (text.find(word) != String::npos) {
+            if (!require_all)
+                return true;
+            } else if (require_all) {
+                return false;
+            }
         }
-    }
-    return gotcha;
+
+  return require_all;
 }
 
 // https://stackoverflow.com/questions/642213/how-to-implement-a-natural-sort-algorithm-in-c
 int natural_compare(const Glib::ustring& left, const Glib::ustring& right);
 
+// Returns a version of text in which all occurrences of words
+// are highlighted using Pango markup
+Glib::ustring highlight_words(const Glib::ustring& text, std::vector<Glib::ustring> words, const Glib::ustring& markup_tag = "b");
+
+Glib::ustring get_accelerator_label(const std::string& accelerator);
 
 } // namespace CtStrUtil
 
@@ -126,6 +160,8 @@ std::string get_font_size_str(const std::string& fontStr);
 
 namespace CtRgbUtil {
 
+// todo: normalized color function, better use RGBA as inner presentation instead of strings
+
 void set_rgb24str_from_rgb24int(guint32 rgb24Int, char* rgb24StrOut);
 
 guint32 get_rgb24int_from_rgb24str(const char* rgb24Str);
@@ -148,12 +184,30 @@ namespace str {
 
 bool startswith(const std::string& str, const std::string& starting);
 
+template<class container>
+bool startswith_any(const Glib::ustring& text, const container& chars_list)
+{
+    for (auto it = std::begin(chars_list); it != std::end(chars_list); ++it)
+        if (str::startswith(text, *it))
+            return true;
+    return false;
+}
+
 bool endswith(const std::string& str, const std::string& ending);
 
 int indexOf(const Glib::ustring& str, const Glib::ustring& lookup_str);
 
 int indexOf(const Glib::ustring& str, const gunichar& uc);
 
+template <typename T, size_t size>
+int indexOf(const std::array<T, size>& array, const T& uc)
+{
+    for (size_t i = 0; i < size; ++i)
+        if (array[i] == uc)
+            return (int)i;
+    return -1;
+}
+
 std::string xml_escape(const std::string& text);
 
 std::string re_escape(const std::string& text);
@@ -166,15 +220,16 @@ int byte_pos_to_symb_pos(const Glib::ustring& text, int byte_pos);
 Glib::ustring swapcase(const Glib::ustring& text);
 
 template<class String>
-String replace(String& subjectStr, const Glib::ustring& searchStr, const Glib::ustring& replaceStr)
+std::string replace(const /* const: func doens't change the source! */ String& subjectStr, const std::string& searchStr, const std::string& replaceStr)
 {
+    Glib::ustring text = subjectStr; // Glib::ustring works with unicode
     size_t pos = 0;
-    while ((pos = subjectStr.find(searchStr, pos)) != std::string::npos)
+    while ((pos = text.find(searchStr, pos)) != std::string::npos)
     {
-        subjectStr.replace(pos, searchStr.size(), replaceStr);
+        text.replace(pos, searchStr.size(), replaceStr);
         pos += replaceStr.size();
     }
-    return subjectStr;
+    return text;
 }
 
 template<class String>
@@ -313,7 +368,9 @@ namespace CtFileSystem {
 // From Slash to Backslash when needed
 std::string get_proper_platform_filepath(std::string filepath);
 
-void copy_file(Glib::ustring from_file, Glib::ustring to_file);
+bool copy_file(Glib::ustring from_file, Glib::ustring to_file);
+
+bool move_file(Glib::ustring from_file, Glib::ustring to_file);
 
 std::string abspath(const std::string& path);
 
@@ -327,4 +384,6 @@ void external_folderpath_open(const std::string& folderpath);
 
 Glib::ustring prepare_export_folder(Glib::ustring dir_place, Glib::ustring new_folder, bool overwrite_existing);
 
+bool rmdir(const std::string& dir);
+
 } // namespace CtFileSystem
