@@ -23,7 +23,10 @@
 
 #include <string>
 #include <list>
+#include <set>
 #include <unordered_map>
+#include <glibmm/ustring.h>
+#include <gtksourceviewmm/buffer.h>
 
 enum class CtYesNoCancel { Yes, No, Cancel };
 
@@ -51,6 +54,17 @@ class CtCodebox;
 class CtMainWin;
 typedef std::pair<CtCodebox*, CtMainWin*>   CtPairCodeboxMainWin;
 
+struct CtListInfo
+{
+    CtListType type = CtListType::None;
+    int        num = -1;   // todo: fix that for bullet and number it has different meanings
+    int        level = -1; // can be filled for NONE to use with shift+return
+    int        aux = -1;
+    int        startoffs = -1;
+
+    operator bool() { return type != CtListType::None; }
+};
+
 struct CtRecentDocRestore
 {
     std::string   exp_coll_str;
@@ -92,3 +106,39 @@ struct CtRecentDocsFilepaths : public CtMaxSizedList<std::string>
 {
     CtRecentDocsFilepaths() : CtMaxSizedList<std::string>{10} {}
 };
+
+
+struct CtStorageNodeState
+{
+    bool upd{false};
+    bool prop{false};
+    bool buff{false};
+    bool hier{false};
+};
+
+struct CtStorageSyncPending
+{
+    bool                                           bookmarks_to_write{false};
+    std::unordered_map<gint64, CtStorageNodeState> nodes_to_write_dict;
+    std::set<gint64>                               nodes_to_rm_set;
+};
+
+class CtAnchoredWidget;
+class CtStorageEntity
+{
+public:
+    CtStorageEntity() = default;
+    virtual ~CtStorageEntity() = default;
+
+    virtual void close_connect() = 0;
+    virtual void reopen_connect() = 0;
+    virtual void test_connection() = 0;
+
+    virtual bool populate_treestore(const Glib::ustring& file_path, Glib::ustring& error) = 0;
+    virtual bool save_treestore(const Glib::ustring& file_path, const CtStorageSyncPending& syncPending, Glib::ustring& error) = 0;
+    virtual void vacuum() = 0;
+
+    virtual Glib::RefPtr<Gsv::Buffer> get_delayed_text_buffer(const gint64& node_id,
+                                                              const std::string& syntax,
+                                                              std::list<CtAnchoredWidget*>& widgets) const = 0;
+};
